/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#include <OpenCLIPER/OpenCLIPERDataModel.hpp>
#include "OpenCLIPER/ProgramConfig.hpp"
using namespace OpenCLIPER;
int main(int argc, char *argv[]) {

  try {
    // Get a new OpenCLIPER app, selecting the CPU as the computing device
    CLapp::PlatformTraits platformTraits;
    CLapp::DeviceTraits deviceTraits;

    ProgramConfig* pProgramConfig = new ProgramConfig(argc, argv);
    auto pConfigTraits = std::dynamic_pointer_cast<ProgramConfig::ConfigTraits>(
        pProgramConfig->getConfigTraits());

    deviceTraits = pConfigTraits->deviceTraits;
    platformTraits = pConfigTraits->platformTraits;

    auto pCLapp = CLapp::create(platformTraits,deviceTraits);

    // Load OpenCL kernel(s) (automatically done for internal kernels during CLapp initialization)
    // Load input data from Matlab file
    std::shared_ptr<Data> pInputKData(new KData(pCLapp, "MRIdataSOS.mat"));

    // Create output with suitable size
    std::shared_ptr<XData> pOutputXData(new XData(pCLapp, std::dynamic_pointer_cast<KData>(pInputKData)));

    // Register input and output in our CL app
    // (data is sent to the computing device automatically)
    // Done automatically by Data subclasses constructors

    // Create new process, set its input/output data sets
    // and bind it to our CL app
    std::unique_ptr<Process> pProcess(new SimpleMRIRecon(pCLapp));
    pProcess->setInput(pInputKData);
    pProcess->setOutput(pOutputXData);

    // Initialize & launch process
    pProcess->init();
    pProcess->launch();

    // Get data back from computing device (automatically done by matlabSave)
    // Save output data
    pOutputXData->matlabSave("outputFrames.mat");
    pOutputXData->show();

    // Clean up
    pProcess.reset(nullptr);
    pInputKData = nullptr;
    pOutputXData = nullptr;
    pCLapp = nullptr;
  } catch (std::exception& e) {
    std::cerr << "Error: " << e.what() << std::endl;
  }
}
