/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#ifndef INCLUDE_OPENCLIPER_MATVARINFO_HPP_
#define INCLUDE_OPENCLIPER_MATVARINFO_HPP_

#include<OpenCLIPER/OpenCLIPERDataModelCommonDefs.hpp>
#include <matio.h>
namespace OpenCLIPER {
/**
 * @brief Class that stores matlab data and its meta-data (matlab class type, matlab data type, etc.)
 *
 */
class MatVarInfo {
public:
    MatVarInfo(enum matio_classes class_type, enum matio_types data_type, void *data, int opt);
    ~MatVarInfo();

    /**
     * @brief Gets class_type class variable
     *
     * @return value class_type class variable
     */
    enum matio_classes getClassType() {return class_type;}

    /**
     * @brief Gets data_type class variable
     *
     * @return value data_type class variable
     */
    enum matio_types getDataType() {return data_type;}

    /**
     * @brief Gets matlab rank (number of dimensions of a matrix)
     *
     * @return value of matlab rank.
     */
    unsigned int getRank() {return rank;}

    size_t* getDims();

    /**
     * @brief Gets pointer to data in matlab format
     *
     * @return pointer to data in matlab format
     */
    void *getData() {return data;}

    /**
     * @brief Gets matlab opt class variable
     *
     * @return the value of matlab opt class variable
     */
    int getOpt() {return opt;}

    void updateDimsAndRank(const std::vector<dimIndexType>* pDimsVectorArg);
private:
    /// enumerated value storing matlab class type
    enum matio_classes class_type;

    /// enumerated value storing matlab data type
    enum matio_types data_type;

    /// matlab rank. Note: rank is a signed int in matio.h, but a negative number of dimensions makes no sense.
    /// Since we do comparisons with unsigned variables all the time, we store it as an unsigned
    unsigned int rank = 0;

    /// vector with dimensions of array
    std::vector<size_t> *pDimsVector = new std::vector<size_t>();

    /// array data in matlab format
    void *data;

    /// matlab options
    int opt;
};
}
#endif // INCLUDE_OPENCLIPER_MATVARINFO_HPP_
