/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#include <OpenCLIPER/OpenCLIPERDataModelCommonDefs.hpp>
#include <OpenCLIPER/NDArray.hpp>
#include <OpenCLIPER/ConcreteNDArray.hpp>
#include <OpenCLIPER/XData.hpp>
#include <OpenCLIPER/SensitivityMapsData.hpp>
#include <OpenCLIPER/SensitivityMapsRMS.hpp>
#include <OpenCLIPER/SamplingMasksData.hpp>
#include <OpenCLIPER/KData.hpp>
#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/processes/FFT.hpp>
#include <OpenCLIPER/processes/ComplexElementProd.hpp>
#include <OpenCLIPER/processes/examples/SimpleMRIRecon.hpp>

#include <LPISupport/Utils.hpp>
#include <LPISupport/SampleCollection.hpp>
#include <LPISupport/InfoItems.hpp>
#include <OpenCLIPER/processes/examples/SimpleMRIReconSOS.hpp>


#include <OpenCLIPER/processes/GroupwiseRegistration.hpp>

#include <OpenCLIPER/processes/groupwiseRegistration/reductions/Reduction7Dto6D.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/Cost6DReduction.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/CostReduction.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/ReductionSum.hpp>

#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/ControlPointsLocation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/CreateBB.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/PermuteBB.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/CreateBBg.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/RepmatBBg.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/PermuteBBg.hpp>

#include <OpenCLIPER/processes/groupwiseRegistration/initialize/InitZero.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/InitZeroRect.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/CopyDataGPU.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/Initdx.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/DataNormalization.hpp>

#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AuxiliarMask.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AuxiliarCost.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Metric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/SumLambdaMetric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Gradient.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/TransformationAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Deformation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/DeformationAdjust.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientInterpolator.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientMetric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientRegularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientJointAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/PermutedV.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/UpdateTransformation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/PermuteTAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/ShiftTAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Regularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Interpolator.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Cost6D.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Regularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientRegularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientWithSmoothTerms.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AdjointInterpolator.hpp>

#include <OpenCLIPER/processes/MotionCompensation.hpp>
#include <OpenCLIPER/processes/AdjointMotionCompensation.hpp>

#include <OpenCLIPER/processes/BlankDataRows.hpp>

#include <OpenCLIPER/processes/nesta/NestaUp.hpp>
#include <OpenCLIPER/processes/nesta/VectorNormalization.hpp>
#include <OpenCLIPER/processes/nesta/TemporalTV.hpp>
// Add class header includes when available
