/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

#ifndef PROCESS_HPP
#define PROCESS_HPP

#include <OpenCLIPER/ProcessCore.hpp>

namespace OpenCLIPER {

/**
 * @brief Class representing common data and behaviour for all the processes in charge of operations on data.
 *
 */
class Process: public ProcessCore {
    public:
	/// Constructors
        Process(): ProcessCore() {}
        Process(std::shared_ptr<CLapp> pCLapp): ProcessCore() { setApp(pCLapp); }
        Process(std::shared_ptr<CLapp> pCLapp, std::shared_ptr<Data> pInputData, std::shared_ptr<Data> pOutputData): ProcessCore(pInputData, pOutputData) { setApp(pCLapp); }

        /// Destructor
        virtual ~Process() { pCLapp = nullptr; }

        virtual void setApp(std::shared_ptr<CLapp> pCLapp);

    protected:
        /**
        * @brief Get value of shared pointer to CLapp process assigned to this object
        * @return shared pointer to CLapp object
        */

        const std::shared_ptr<CLapp> getApp() const {
            if(pCLapp!=nullptr)
                return pCLapp;
            else throw std::runtime_error("Process::getApp() called before setApp()");
        }

    private:
        /// Smart shared pointer to object with OpenCL context
        std::shared_ptr<CLapp> pCLapp = nullptr;
};
} // namespace OpenCLIPER
#endif // PROCESS_HPP
