/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodrguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martn Gonzlez,
 *                    Elisa Moya Sez,
 *                    Marcos Martn Fernndez and
 *                    Carlos Alberola Lpez
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicacin
 *  Universidad de Valladolid
 *  Paseo de Beln 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
/*
 * FFT.hpp
 *
 *  Created on: 23 de nov. de 2016
 *      Author: fedsim
 */

#ifndef INCLUDE_OPENCLIPER_FFT_HPP
#define INCLUDE_OPENCLIPER_FFT_HPP

#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/processes/ScalarMultiply.hpp>
#include <clFFT.h>

#ifdef HAVE_ROCFFT
    #include <rocfft.h>
#endif

// CLFFT_SINGLE_FAST and CLFFT_DOUBLE_FAST are apparently not
// implemented in clFFT right now
#ifdef DOUBLE_PREC
    #define OPENCLIPER_CLFFT_PRECISION CLFFT_DOUBLE
    #define OPENCLIPER_ROCFFT_PRECISION rocfft_precision_double
#else
    #define OPENCLIPER_CLFFT_PRECISION CLFFT_SINGLE
    #define OPENCLIPER_ROCFFT_PRECISION rocfft_precision_single
#endif


namespace OpenCLIPER {
/**
 * @brief Process class for the FFT operation on a image
 *
 */
class FFT : public Process {
    public:
        /// Enumerated type with direction of FFT
	enum Direction {
            /// Forward direction
            FORWARD=CLFFT_FORWARD,
            /// Backward direction
            BACKWARD=CLFFT_BACKWARD
        };

        /// Parameters related to kernel execution
        struct LaunchParameters:Process::LaunchParameters {
            /// direction of FFT
            Direction dir;
            /// constructor
            LaunchParameters(Direction d=Direction::FORWARD):dir(d) {}
        };

        /// Constructors
        FFT(): Process() {}
        FFT(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {}
        FFT(std::shared_ptr<OpenCLIPER::CLapp> pCLapp, std::shared_ptr<Data> pIn, std::shared_ptr<Data> pOut): Process(pCLapp,pIn,pOut) {}
        ~FFT();

        // Methods
	void init();
	void launch(ProfileParameters profileParameters);

    private:
        /// handle to FFT plan (FFT configuration)
        clfftPlanHandle clPlanHandle;

#ifdef HAVE_ROCFFT
        rocfft_plan     rocPlanHandleFW;
        rocfft_plan     rocPlanHandleBW;

        rocfft_plan_description rocfftPlanDescription;

        void*           rocWorkBufferFW;
        size_t          rocWorkBufferBytesFW;
        void*           rocWorkBufferBW;
        size_t          rocWorkBufferBytesBW;

        rocfft_execution_info rocExecInfoFW;
        rocfft_execution_info rocExecInfoBW;

        std::shared_ptr<ScalarMultiply> scalarMultiply;
        std::shared_ptr<ScalarMultiply::LaunchParameters> scalarMultiplyLP;
#endif

};

} /* namespace OpenCLIPER */

#endif /* INCLUDE_OPENCLIPER_FFT_HPP */
