/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

#ifndef INCLUDE_OPENCLIPER_GROUPWISEREGISTRATION_HPP_
#define INCLUDE_OPENCLIPER_GROUPWISEREGISTRATION_HPP_
#include <OpenCLIPER/CLapp.hpp>
#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/ConcreteNDArray.hpp>
#include <OpenCLIPER/XData.hpp>
#include <OpenCLIPER/Data.hpp>

#include <OpenCLIPER/processes/groupwiseRegistration/reductions/Reduction7Dto6D.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/Cost6DReduction.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/CostReduction.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/reductions/ReductionSum.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/ControlPointsLocation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/CreateBB.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/PermuteBB.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/CreateBBg.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/RepmatBBg.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/creabspline/PermuteBBg.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/InitZero.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/InitZeroRect.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/CopyDataGPU.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/Initdx.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/initialize/DataNormalization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AuxiliarMask.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AuxiliarCost.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Metric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/SumLambdaMetric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Gradient.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/TransformationAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Deformation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/DeformationAdjust.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientInterpolator.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientMetric.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientRegularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientJointAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/PermutedV.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/UpdateTransformation.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/PermuteTAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/ShiftTAux.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Regularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Interpolator.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Cost6D.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/Regularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientRegularization.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/GradientWithSmoothTerms.hpp>
#include <OpenCLIPER/processes/groupwiseRegistration/optimizer/AdjointInterpolator.hpp>

#include <algorithm>
#include <numeric> // Needed for gcc 7 (default compiler in Ubuntu 18.04)
#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


namespace OpenCLIPER {

/**
 * @brief Structure with arguments obtained in Groupwise Registration, needed in Motion Compensation
 *
 */
  struct ArgumentsMotionCompensation {
    std::shared_ptr<Data> xnData;
    std::shared_ptr<Data> TData;
    cl::Buffer xobj;
    int* bound_box;
  };

/**
 * @brief Struct with host memory objects used in creabspline()
 *
 */
  struct Bspline {
    uint nt;
    int Dp[2];
    uint E;
    float c[2];
    float rp[2];
    int C[2][2];
    int *coef;
    int *coefg;
    std::shared_ptr<Data> BBgall;
    std::shared_ptr<Data> BBall;
  };

/**
 * @brief Struct with the corresponding weights applied to the smoothness/regularization terms.
 *
 */
  struct Terms {
    std::vector<float> lambda; //!< Smoothness terms (1st spatial, 2nd spatial, 1st temporal, 2nd temporal)
  };

/**
 * @brief Struct with the parameters used in gradient descent optimizer.
 *
 */
  struct ParametersGD {
    uint nmax; //!< Maximum number of optimization loop iterations
    float et; //!< Transformation norm threshold (in pixels)
    float eh; //!< Metric variation threshold (e.g. 0.005 => 0.5% of initial metric)
  };

/**
 * @brief Struct where data dimensions are stored.
 *
 */
struct Dimensions{
    // 2D
    std::vector<cl_uint> V; //!< V dimensions (2-Dimensions)
    std::vector<cl_uint> X; //!< X dimensions (2-Dimensions)
    std::vector<cl_uint> mean3D; //!< mean3D dimensions (2-Dimensions)

    // 3D
    std::vector<cl_uint> I; //!< I dimensions (3-Dimensions)
    std::vector<cl_uint> x; //!< x dimensions (3-Dimensions)
    std::vector<cl_uint> u; //!< u dimensions (3-Dimensions)
    std::vector<cl_uint> pu; //!< pu dimensions (3-Dimensions)
    std::vector<cl_uint> coef; //!< coef dimensions (3-Dimensions)
    std::vector<cl_uint> coefg; //!< coefg dimensions (3-Dimensions)
    std::vector<cl_uint> mean4D; //!< mean4D dimensions (3-Dimensions)

    // 4D
    std::vector<cl_uint> BBAux; //!< BBAux dimensions (4-Dimensions)
    std::vector<cl_uint> BBAux1; //!< BBAux1 dimensions (4-Dimensions)
    std::vector<cl_uint> dH; //!< dH dimensions (4-Dimensions)
    std::vector<cl_uint> T; //!< T dimensions (4-Dimensions)
    std::vector<cl_uint> Dif; //!< Dif dimensions (4-Dimensions)
    std::vector<cl_uint> xn; //!< xn dimensions (4-Dimensions)
    std::vector<cl_uint> d; //!< d dimensions (4-Dimensions)

    // 5D
    std::vector<cl_uint> auxBB; //!< auxBB dimensions (5-Dimensions)
    std::vector<cl_uint> auxBBg; //!< auxBBg dimensions (5-Dimensions)
    std::vector<cl_uint> Aux; //!< Aux dimensions (5-Dimensions)
    std::vector<cl_uint> dtau; //!< dtau dimensions (5-Dimensions)

    // 6D
    std::vector<cl_uint> dV; //!< dV dimensions (6-Dimensions)
    std::vector<cl_uint> TAux; //!< TAux dimensions (6-Dimensions)

    // 7D
    std::vector<cl_uint> aux1BBg; //!< aux1BBg dimensions (7-Dimensions)
    std::vector<cl_uint> BB; //!< BB dimensions (7-Dimensions)
    std::vector<cl_uint> BBg; //!< BBg dimensions (7-Dimensions)
    std::vector<cl_uint> dtheta; //!< dtheta dimensions (7-Dimensions)
  };


/**
 * @brief Process class to perform the groupwise registration, with metric VI (variance of intensity).
 *
 */
class GroupwiseRegistration: public Process {
  public:
    struct LaunchParameters:Process::LaunchParameters {
        std::string baseInputFileName;
        ArgumentsMotionCompensation* argsMC;
        uint nmax; //!< Maximum number of optimization loop iterations
        float et; //!< Transformation norm threshold (in pixels)
        float eh; //!< Metric variation threshold (e.g. 0.005 => 0.5% of initial metric)
        std::vector<float> lambda; //!< Smoothness terms (1st spatial, 2nd spatial, 1st temporal, 2nd temporal)
        uint radius;
        uint E;
        int* Dp;
        float W;
        bool flagW;
        std::shared_ptr<Data> TData;

        Bspline* ts;
        int currentMotionIter;

        LaunchParameters(ArgumentsMotionCompensation* args, uint nmax, float et, float eh, std::vector<float> lambda, uint radius, uint E, int* Dp, float W, bool flagW, std::shared_ptr<Data> TData, Bspline* ts, int iter):
        argsMC(args), nmax(nmax), et(et), eh(eh), lambda(lambda), radius(radius), E(E), Dp(Dp), W(W), flagW(flagW), TData(TData), ts(ts), currentMotionIter(iter) {}
    };

    GroupwiseRegistration(): Process() {}
    GroupwiseRegistration(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};

    void init();
    const std::shared_ptr<LaunchParameters> getLaunchParameters();
    void launch(ProfileParameters profileParameters = {false, 1});
    ~GroupwiseRegistration();

    // Getters
    /** Returns struct containing the dimensions of every memory object used in GroupwiseRegistration
     * @returns dimensions of every memory object used in GroupwiseRegistration
     */
    Dimensions getDimensions() {return dim;};
    /** Returns struct containig weights for every regularization term
     * @returns weights for every regularization term
     */
    Terms getTerms() {return term;};
    /** Returns struct containing parameters needed in Gradient Descent Algorithm
     * @returns parameters needed in Gradient Descent Algorithm
     */
    ParametersGD getParametersGD() {return param;};
    /** Returns vector with images size
     * @returns images size
     */
    std::vector<cl_uint> getDataSize() {return dataSize; };

    // Setters
    void setDimensions(const Bspline *ts, const cl_uint longr1, const cl_uint longr2, const cl_uint longr1margin, const cl_uint longr2margin, const unsigned int select);
    void setTerms(const std::vector<float> lambda);
    void setParametersGD(const unsigned int paramNmax, const float paramEt, const float paramEh);
    void setDataSize(std::vector<cl_uint> dataSize);

  private:
    std::vector<cl_uint> dataSize;
    Dimensions dim; // Arrays dimensions
    Terms term; // Lambda terms
    ParametersGD param; // Parameters for Gradient Descent Algorithm


    std::unique_ptr<Process> pControlPoints;
    std::unique_ptr<Process> pCreateBB;
    std::unique_ptr<Process> pPermuteBB;
    std::unique_ptr<Process> pCreateBBg;
    std::unique_ptr<Process> pRepmatBBg;
    std::unique_ptr<Process> pPermuteBBg;

    std::unique_ptr<Process> pInitZero;
    std::unique_ptr<Process> pInitIT;
    std::unique_ptr<Process> pMetric;
    std::unique_ptr<Process> pSumLambdaMetric;
    std::unique_ptr<Process> pGradient;
    std::unique_ptr<Process> pPermutedV;
    std::unique_ptr<Process> pUpdateTransformation;
    std::unique_ptr<Process> pCost6D;
    std::unique_ptr<Process> pGradientMetric;
    std::unique_ptr<Process> pGradientJointAux;
    std::unique_ptr<Process> pGradientWithSmoothTerms;
    std::unique_ptr<Process> pReduction7Dto6D;
    std::unique_ptr<Process> pReduction6Dto5D;
    std::unique_ptr<Process> pReduction5Dto4D;
    std::unique_ptr<Process> pAuxiliarMask;
    std::unique_ptr<Process> pAuxiliarCost;
    std::unique_ptr<Process> pCost6DReduction;
    std::unique_ptr<Process> pCostReduction;
    std::unique_ptr<Process> pTransformationAux;
    std::unique_ptr<Process> pDeformation;
    std::unique_ptr<Process> pDeformationAdjust;
    std::unique_ptr<Process> pInitdx;
    std::unique_ptr<Process> pGradientInterpolator;
    std::unique_ptr<Process> pPermuteTAux;
    std::unique_ptr<Process> pShiftTAux;
    std::unique_ptr<Process> pRegularization;
    std::unique_ptr<Process> pInterpolator;
    std::unique_ptr<Process> pGradientRegularization;

    float sumLambda();
    int setMask(int *X, int *bound_box, const int radius);
    void creabspline(Bspline* ts, int* x, int *bound_box, int Dp[], int E, bool profilingEnabled);
    void optimizer(std::shared_ptr<Data> inputData, std::shared_ptr<Data> TData, int *x, Bspline *ts, int *X, float Wn, int *bound_box, bool flagW, ArgumentsMotionCompensation *argsMC, bool profilingEnabled);
    void evolution(float *Tdif, float *Hdif, float *Wn, realType* Dif, realType* H, int iter, bool flagW);
    bool stopCondition(const float p1, const float p3, const float p2, const float p4, const int p5, const int p6);
  };



  /**
   * @brief B-spline order 0
   * @param[in] x
   * @return Result of the calculation
   */
  float bspline0(float x);

  /**
   * @brief First order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float bspline1(float x);

  /**
   * @brief Second order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float bspline2(float x);

  /**
   * @brief Third order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float bspline3(float x);

  /**
   * @brief n order B-spline
   * @param[in] x
   * @param[in] n B-spline order
   * @return Result of the calculation
   */
  float bsplineN(float x, int n);

  /**
   * @brief First derivative of first order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float b1spline1(float x);

  /**
   * @brief First derivative of second order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float b1spline2(float x);

  /**
   * @brief First derivative of third order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float b1spline3(float x);

  /**
   * @brief First derivative of n order B-spline
   * @param[in] x
   * @param[in] n B-spline order
   * @return Result of the calculation
   */
  float b1splineN(float x,int n);

  /**
   * @brief Second derivative of second order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float b2spline2(float x);

  /**
   * @brief Second derivative of third order B-spline
   * @param[in] x
   * @return Result of the calculation
   */
  float b2spline3(float x);

  /**
   * @brief Second derivative of second order B-spline
   * @param[in] x
   * @param[in] n B-spline order
   * @return Result of the calculation
   */
  float b2splineN(float x, int n);

}

#endif /* INCLUDE_OPENCLIPER_GROUPWISEREGISTRATION_HPP_ */
