/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

#ifndef INCLUDE_OPENCLIPER_MOTIONCOMPENSATION_HPP_
#define INCLUDE_OPENCLIPER_MOTIONCOMPENSATION_HPP_

#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/processes/GroupwiseRegistration.hpp>

namespace OpenCLIPER {

/**
 * @brief Process class to apply the motion compensation obtained by the groupwise registration.
 *        From the groupwise registration we obtain the original mesh, the new mesh with the pixelwise
 *        displacements and the bounding box where it has to be applied.
 *        It can be used in NESTA reconstruction to refine a solution.
 *
 */
class MotionCompensation : public Process {
    public:
	struct LaunchParameters:Process::LaunchParameters {
	    ArgumentsMotionCompensation* argsMC;

            LaunchParameters(ArgumentsMotionCompensation* args): argsMC(args) {}
	};

        MotionCompensation(): Process() {}
        MotionCompensation(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};
	void launch(ProfileParameters profileParameters = {false, 1});
};
}
#endif /* INCLUDE_OPENCLIPER_MOTIONCOMPENSATION_HPP_ */
