/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#ifndef INCLUDE_OPENCLIP_SIMPLEMRIRECON_HPP_
#define INCLUDE_OPENCLIP_SIMPLEMRIRECON_HPP_

#include <OpenCLIPER/CLapp.hpp>
#include <OpenCLIPER/XData.hpp>
#include <OpenCLIPER/KData.hpp>
#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/processes/FFT.hpp>
#include <OpenCLIPER/processes/ComplexElementProd.hpp>
#include <OpenCLIPER/processes/XImageSum.hpp>
namespace OpenCLIPER {

/// @brief Class that makes a simple MRI reconstruction of a group of k-images captured by several coils.
class SimpleMRIRecon: public Process {
    public:
	
	/**
	 * @brief Default constructor.
	 * 
	 * It calls the superclass constructor.
	 * 
	 */
        SimpleMRIRecon(): Process() {}
        SimpleMRIRecon(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};
        ~SimpleMRIRecon();

        void init();
        void launch(ProfileParameters profileParameters);

    private:
        /// Pointer to Process subclass in charge of obtaining the inverse FFT of a group of k-images
        std::unique_ptr<Process> pProcInvFFT;
        /// Pointer to Process subclass in charge of multiplying every x-image by the sensitivity map of the coil used to capture it
        std::unique_ptr<Process> pProcSensMapProd;
        /// Pointer to Process subclass in charge of adding images captured from all the coils at the same time frame
        std::unique_ptr<Process> pProcAddXImages;
};

} //namespace OpenCLIPER

#endif /* INCLUDE_SIMPLEMRIRECON_HPP_ */
