/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#ifndef INCLUDE_OPENCLIPER_DEFORMATION_HPP_
#define INCLUDE_OPENCLIPER_DEFORMATION_HPP_
#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/CLapp.hpp>


namespace OpenCLIPER {

/**
 * @brief Process class to apply the BSpline transformation given by the 
 *        control points to the rest of the pixels in the region of interest.
 *        This results in pixelwise displacements.
 *       
 */
class Deformation: public Process {
    public:
        
        struct LaunchParameters:Process::LaunchParameters {
            std::shared_ptr<Data> BBallData;
            int* bound_box;
            
            LaunchParameters(std::shared_ptr<Data> BBallData, int* bound_box):
            BBallData(BBallData), bound_box(bound_box) {}
        };
        
        Deformation(): Process() {}
        Deformation(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};

        void init();
        void launch(ProfileParameters profileParameters = {false, 1});
        
    private:
        cl::Buffer r1obj;
        cl::Buffer r2obj;
        enum BBx { BB=0, BB1=1, BB2=2, BB11=3};
};
  
} /* namespace OpenCLIPER */

#endif /* INCLUDE_OPENCLIPER_TRANSFORMATION_HPP_ */
