/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#ifndef INCLUDE_OPENCLIPER_GRADIENTINTERPOLATOR_HPP_
#define INCLUDE_OPENCLIPER_GRADIENTINTERPOLATOR_HPP_
#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/CLapp.hpp>

namespace OpenCLIPER {

/**
 * @brief Process class to apply the interpolator to the image gradient, to deform it
 *        in the way given by the transformation.
 *       
 */
class GradientInterpolator: public Process {
    public:
        
        struct LaunchParameters:Process::LaunchParameters {
            std::shared_ptr<Data> dallData;
            cl::Buffer xobj;
            int* bound_box;
            
            LaunchParameters(std::shared_ptr<Data> dallData, cl::Buffer& x, int* bound_box): dallData(dallData), xobj(x), bound_box(bound_box) {}
        };
        
        GradientInterpolator(): Process() {}
        GradientInterpolator(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};

        void init();
        void launch(ProfileParameters profileParameters = {false, 1});
        
    private:
        cl::Buffer r1marginobj;
        cl::Buffer r2marginobj;
        
        enum d { d1=0, d2=1};
        
};
  
} /* namespace OpenCLIPER */

#endif /* INCLUDE_OPENCLIPER_GRADIENTINTERPOLATOR_HPP_ */
