/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

#ifndef INCLUDE_LPIMRI_NESTAUP_HPP_
#define INCLUDE_LPIMRI_NESTAUP_HPP_

#include <OpenCLIPER/Process.hpp>
#include <OpenCLIPER/processes/FFT.hpp>
#include <OpenCLIPER/processes/ComplexElementProd.hpp>
#include <OpenCLIPER/processes/XImageSum.hpp>
#include <OpenCLIPER/processes/BlankDataRows.hpp>
#include <OpenCLIPER/processes/nesta/TemporalTV.hpp>
#include <OpenCLIPER/processes/nesta/VectorNormalization.hpp>
#include <OpenCLIPER/processes/GroupwiseRegistration.hpp>
#include <OpenCLIPER/processes/MotionCompensation.hpp>
#include <OpenCLIPER/processes/AdjointMotionCompensation.hpp>
#include <clblast_c.h>
#include <algorithm>

namespace OpenCLIPER {
    
/**
 * @brief Process class to apply Nesta algorithm
 *
 */

class NestaUp : public Process {
    public:
        struct LaunchParameters:Process::LaunchParameters {
            float lambda_i;
            float mu_f;
            float La;
            uint maxIntIter;
            float tolVar; //Definido segun DemoUP
            int verbose;
            unsigned int maxIter;
            uint stoptest;
            ArgumentsMotionCompensation* argsMC;

            LaunchParameters(float lambda_i, float mu_f, float La, uint maxIntIter, float tolVar, int verbose, int maxIter, uint stoptest, ArgumentsMotionCompensation* args): lambda_i(lambda_i), mu_f(mu_f), La(La), maxIntIter(maxIntIter), tolVar(tolVar), verbose(verbose),
            maxIter(maxIter), stoptest(stoptest), argsMC(args) {}
        };

        NestaUp(): Process() {}
        NestaUp(std::shared_ptr<OpenCLIPER::CLapp> pCLapp): Process(pCLapp) {};

        void init();
        void launch(ProfileParameters profileParameters = {false, 1});
        ~NestaUp();


    private:
        std::unique_ptr<Process> pFFTOutOfPlace;
        std::unique_ptr<Process> pDataAndSensitivityMapsProduct;
        std::unique_ptr<Process> pDataAndSamplingMasksProduct;
        std::unique_ptr<Process> pXImagesAllCoilSameFrameAddition;
        std::unique_ptr<Process> pTemporalTV;
        std::unique_ptr<Process> pTemporalTVt;
        std::unique_ptr<Process> pVectorNormalization;
        std::unique_ptr<Process> pMotionCompensation;
        std::unique_ptr<Process> pAdjointMotionCompensation;
        std::unique_ptr<Process> pCopy;

        void operatorA(std::shared_ptr<Data> inputData, std::shared_ptr<SensitivityMapsData> sensitivityMapsData, std::shared_ptr<SamplingMasksData> samplingMasksData, std::shared_ptr<Data> outputData,std::shared_ptr<Data> auxDataFFT, bool profilingEnabled);
        void operatorAt(std::shared_ptr<Data> inputData, std::shared_ptr<SensitivityMapsData> sensitivityMapsData, std::shared_ptr<Data> outputData, std::shared_ptr<Data> auxDataFFT, bool profilingEnabled);
        void operatorU(std::shared_ptr<Data> inputData, std::shared_ptr<Data> outputData, std::shared_ptr<Data> auxiliarDataMC, ArgumentsMotionCompensation* argsMC, bool profilingEnabled);
        void operatorUt(std::shared_ptr<Data> inputData, std::shared_ptr<Data> outputData, std::shared_ptr<Data> auxiliarDataMC, ArgumentsMotionCompensation* argsMC, bool profilingEnabled);
        float myNormest(uint rows, uint cols, uint NumFrames,  ArgumentsMotionCompensation* argsMC, bool profilingEnabled);
};

}

#endif /* INCLUDE_LPIMRI_NESTAUP_HPP_ */
