/*
 * RCS/CVS version control info
 * $Id: reduce_kernel.cl,v 1.2 2016/11/02 12:34:19 manrod Exp $
 * $Revision: 1.2 $
 * $Date: 2016/11/02 12:34:19 $
 */

#include <OpenCLIPER/kernels/hostKernelFunctions.h>
//#pragma OPENCL_EXTENSION cl_amd_printf
//#pragma OPENCL EXTENSION cl_amd_printf : enable

//#define DEBUGKERNEL 1

inline void createConstantVectors(VECTORDATATYPE* pZeroVector) {
    realType* pZeroVectorAsArray = (realType*) pZeroVector;
    for (uint i = 0; i < VECTORDATATYPESIZE; i++) {
        pZeroVectorAsArray[i] = 0.0;
    }
}

inline void blankComplexVector(global realType* pInputBuffer, const VECTORDATATYPE* pZeroVector) {
    uint offsetInNumberOfElements = 0;
    VECTORDATATYPE op1;
    op1 = VLOADN(offsetInNumberOfElements, pInputBuffer);
    op1 = op1 * (*pZeroVector);
    VSTOREN(op1, offsetInNumberOfElements, pInputBuffer);
}

// Only in-place blank allowed!!
__kernel void blankDataRows_kernel(global realType* pInputBuffer, global const uint* pSamplingMaskBuffer,
                                   uint frameId) {
    uint inputOffsetFrameId, inputOffsetCoilId, inputOffsetCoilAndRowId, inputIndexRealPartElement;
    uint coilId, samplingMaskElementId, rowId, realElementColumnId;
    uint numCoils, numSamplingMaskElements, realElementsNumColumns;
    uint samplingMaskDataElementPos;
    VECTORDATATYPE zeroVector;

    createConstantVectors(&zeroVector);

    uint frameIds[1] = {frameId};
    numCoils = getNumCoils(pInputBuffer);
    coilId = get_global_id(2);
    while (coilId < numCoils) {
        inputOffsetFrameId = frameId * getTemporalDimStride(pInputBuffer, 0, getNDArray1DIndex(pInputBuffer, coilId, frameIds));
        inputOffsetCoilId = coilId * getCoilStride(pInputBuffer, getNDArray1DIndex(pInputBuffer, coilId, frameIds));
        realElementsNumColumns = getSpatialDimSize(pInputBuffer, COLUMNS, 0) * 2;
        realElementColumnId = get_global_id(0) * VECTORDATATYPESIZE;
        while (realElementColumnId < realElementsNumColumns) {
            samplingMaskElementId = get_global_id(1);
            numSamplingMaskElements = getSpatialDimSize(pSamplingMaskBuffer, COLUMNS, getNDArray1DIndex(pSamplingMaskBuffer, 0, frameIds));
            while (samplingMaskElementId < numSamplingMaskElements) {
                uint NDArray1DIndex = getNDArray1DIndex(pSamplingMaskBuffer, 0, frameIds);
                 samplingMaskDataElementPos = samplingMaskElementId *
                    getSpatialDimStride(pSamplingMaskBuffer, COLUMNS, NDArray1DIndex) +
                    frameId * getTemporalDimStride(pSamplingMaskBuffer, 0, NDArray1DIndex);
                rowId = pSamplingMaskBuffer[samplingMaskDataElementPos];
                inputOffsetCoilAndRowId = inputOffsetCoilId + rowId * getSpatialDimStride(pInputBuffer, ROWS, 0);
                inputIndexRealPartElement = inputOffsetFrameId + inputOffsetCoilAndRowId + realElementColumnId;
                blankComplexVector(&(pInputBuffer[inputIndexRealPartElement]), &zeroVector);
                samplingMaskElementId += get_global_size(1);
            }
            realElementColumnId += get_global_size(0) * VECTORDATATYPESIZE;
        }
        coilId += get_global_size(2);
    }
}
