/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

/*! \file temporalTV.cl
 *	\brief File containing every kernel about temporal total variation
 *  \date May 15, 2018
 *  \author Elisa Moya Saez
 */


__kernel void operator_tTV(__global float2* in, __global float2* out){
    
    int i=get_global_id(0); // rowID
    int j=get_global_id(1); // colID
    
    uint cols=getSpatialDimSize(in, COLUMNS, 0);
    uint rows=getSpatialDimSize(in, ROWS, 0);
    
    uint idx1 = j*cols+i;
    uint idx2 = idx1;

    uint numFrames = getTemporalDimSize(in, 0);
    
    //Primer frame diferente
    out[idx1]=in[idx1]-in[idx1+(numFrames-1)*cols*rows];
    
    
    for(uint k=1; k<numFrames; k++){
        
        idx1+=getTemporalDimStride(in, 0, 0)/2; // Dividimos entre 2 porque devuelve el nmero de elementos y estamos trabajando con float2 (complejos)
        
        out[idx1]=in[idx1]-in[idx2];
        
        idx2+=getTemporalDimStride(in, 0, 0)/2;
    }
}

__kernel void operator_tTVadj(__global float2* in, __global float2* out){
    
    int i=get_global_id(0);
    int j=get_global_id(1);
    
    uint cols=getSpatialDimSize(in, COLUMNS, 0);
    uint rows=getSpatialDimSize(in, ROWS, 0);
    uint idx1 = j*cols+i;
    uint idx2 = idx1;
    
    uint numFrames = getTemporalDimSize(in, 0);
    
    for(uint k=0; k<numFrames-1; k++){
        
        idx2+=getTemporalDimStride(in, 0, 0)/2; 
        
        out[idx1]=in[idx1]-in[idx2];
        
        idx1+=getTemporalDimStride(in, 0, 0)/2; 
    }
    
     //ultimo frame diferente
    out[idx1]=in[idx1]-in[j*cols+i];
    //out[idx+(numFrames-1)*cols*rows]=in[idx+(numFrames-1)*cols*rows]-in[idx];
   
}

