/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */

#include <OpenCLIPER/processes/AdjointMotionCompensation.hpp>
#include <libgen.h>

namespace OpenCLIPER {

  void AdjointMotionCompensation::launch(ProfileParameters profileParameters) {
    auto pLP=std::dynamic_pointer_cast<LaunchParameters>(pLaunchParameters);

    infoItems.addInfoItem("Title", "AdjointMotionCompensation info");
    startProfiling(profileParameters.profilingEnabled);
    try {

      std::unique_ptr<Process> pCopy(new CopyDataGPU(getApp()));
      pCopy->setInput(getInput());
      pCopy->setOutput(getOutput());
      pCopy->init();
      pCopy->launch(profileParameters.profilingEnabled);
      pCopy.reset(nullptr);

      std::unique_ptr<Process> pInitZeroRect(new InitZeroRect(getApp()));
      pInitZeroRect->setInput(getInput());
      pInitZeroRect->setOutput(getOutput());
      auto parinitZeroRect = std::make_shared<OpenCLIPER::InitZeroRect::LaunchParameters>(pLP->argsMC->bound_box);
      pInitZeroRect->setLaunchParameters(parinitZeroRect);
      pInitZeroRect->init();
      pInitZeroRect->launch(profileParameters.profilingEnabled);
      pInitZeroRect.reset(nullptr);

      std::unique_ptr<Process> pAdjointProcess(new AdjointInterpolator(getApp()));
      pAdjointProcess->setInput(getInput());
      pAdjointProcess->setOutput(getOutput());
      auto parAdjoint = std::make_shared<OpenCLIPER::AdjointInterpolator::LaunchParameters>(pLP->argsMC->xnData, pLP->argsMC->xobj,
											    pLP->argsMC->bound_box);
      pAdjointProcess->setLaunchParameters(parAdjoint);
      pAdjointProcess->init();
      pAdjointProcess->launch(profileParameters.profilingEnabled);
      pAdjointProcess.reset(nullptr);

      stopProfiling(profileParameters.profilingEnabled);
    } catch (cl::Error& err) {
      /////////////////////////////////////////////////////////////////
      // Catch OpenCL errors and print log if it is a build error
      /////////////////////////////////////////////////////////////////
      std::cerr << "ERROR: " << err.what() << " (" << err.err() << ", "
      << OpenCLIPER::CLapp::getOpenCLErrorCodeStr(err.err()) << ")"
      << "\tfile: " << __FILE__ << "\tline: " << __LINE__
      << std::endl;
      throw;
    } catch (std::exception& e) {
      std::cout << "Error: " << e.what() << std::endl;
    }
  }
}
