/* Copyright (C) 2018 Federico Simmross Wattenberg,
 *                    Manuel Rodríguez Cayetano,
 *                    Javier Royuela del Val,
 *                    Elena Martín González,
 *                    Elisa Moya Sáez,
 *                    Marcos Martín Fernández and
 *                    Carlos Alberola López
 *
 * This file is part of OpenCLIPER.
 *
 * OpenCLIPER is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * OpenCLIPER is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenCLIPER; If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Contact:
 *
 *  Federico Simmross Wattenberg
 *  E.T.S.I. Telecomunicación
 *  Universidad de Valladolid
 *  Paseo de Belén 15
 *  47011 Valladolid, Spain.
 *  fedsim@tel.uva.es
 */
#include <OpenCLIPER/processes/examples/SimpleMRIReconSOS.hpp>
#include <LPISupport/InfoItems.hpp>
#include <sys/time.h>

namespace OpenCLIPER {
void SimpleMRIReconSOS::init() {
    pProcInvFFT.reset(new FFT(getApp()));
    pProcInvFFT->setInput(getInput());
    pProcInvFFT->setOutput(getInput());
    pProcInvFFT->init();

    pRSoS.reset(new RSoS(getApp()));
    pRSoS->init();

}

SimpleMRIReconSOS::~SimpleMRIReconSOS() {
    pProcInvFFT.reset(nullptr);
    pRSoS.reset(nullptr);

}

void SimpleMRIReconSOS::launch(ProfileParameters profileParameters) {


    // Step 0: Inverse FFT of initial KData in place
    auto launchParmsInvFFT = std::make_shared<FFT::LaunchParameters>(FFT::BACKWARD);
    pProcInvFFT->setLaunchParameters(launchParmsInvFFT);

    struct timeval t0, t1;

    gettimeofday(&t0, 0);
    pProcInvFFT->launch(profileParameters);

    gettimeofday(&t1, 0);

    long elapsed = (t1.tv_sec-t0.tv_sec)*1000000 + t1.tv_usec-t0.tv_usec;

    FILE * fp1 = fopen ( "TiemposEjecucionFFT.txt", "a+" );
    fseek(fp1, 0, SEEK_END);
    fprintf(fp1, "%ld\n", elapsed);
    fclose(fp1);

    // FFT in-place, save results of first kernel
    std::dynamic_pointer_cast<KData>(getInput())->matlabSave("Temporal_IFFT_MRIReconSOS.mat", SyncSource::BUFFER_ONLY);

    // Step 1: Root of Sum of Squares
    pRSoS->setInput(getInput());
    pRSoS->setOutput(getOutput());

    gettimeofday(&t0, 0);

    pRSoS->launch(profileParameters);

    getApp()->getCommandQueue().finish();

    gettimeofday(&t1, 0);//Elisa

    elapsed = (t1.tv_sec-t0.tv_sec)*1000000 + t1.tv_usec-t0.tv_usec; //Elisa

    FILE * fp2 = fopen ( "TiemposEjecucionRSS.txt", "a+" );
    fseek(fp2, 0, SEEK_END);
    fprintf(fp2, "%ld\n", elapsed);
    fclose(fp2);
}
}
